TEVWH_ELF_OFF target_new_entry_addr(const Target* t)
{
  TRACE_DEBUG(-1, "target_new_entry_addr\n");

  /* matches with aligned_end_of_cs */
  return ALIGN_UP(t->phdr_code->p_vaddr + t->phdr_code->p_filesz);
}
