		.section .text
		.globl _start
start:		push	$0		# replace with original entry address
		pushf
		pusha

		push	$4
		pop	%eax		# eax = 4 = write(2)
		push	$3		# nbytes = three characters
		push	$0x08048001	# buf = magic address
		push	$1		# fd = stdout
		push	%eax		# dummy
		int	$0x80

		popa
		popf
		ret

		push	$start + 1	# dummy instruction to specify ofs
