		BITS 32

		push	ebp
		mov	ebp,esp
		sub	esp,byte 0xc

wrapper:	; replace -1 with address of original host code
		mov	eax,dword -1
		xchg	eax,[ebp]
		sub	ebp,byte 4
		mov	[ebp],eax
		align 8

		; dummy instruction to specify offset
		push    byte wrapper + 1
