#include "src/one_step_closer/action.inc"
#include "src/one_step_closer/print_summary.inc"
#include "src/one_step_closer/open_dst.inc"
#include "src/one_step_closer/is_elf.inc"

#include "patch_entry_addr.inc"

