#include <sys/syscall.h>
		
		.section ".text"
		.global _start
_start:		mov	SYS_write, %g1		! write
		mov	1, %o0			! fd=1
		set	0x10001, %o1		! buf=0x10001
		mov	3, %o2			! count=3
		ta	8

		mov	SYS_exit, %g1		! exit
		mov	%g0, %o0		! status=0
		ta	8
