int print_summary(int stat[])
{
  print_errno(-1, "files=%d; ok=%d; det_page=%d; det_align=%d; "
    "min=0x%04x; max=0x%04x\n",
    stat[0], stat[1], stat[0] - stat[1], stat[2], stat[3], stat[4]
  );
  return 0;
}
