int get_relocate_ofs(void)
{
  int result;
  __asm__(
    "call   delta         ;"
    "delta:                "
    "pop    %%eax         ;"
    "sub    $(delta),%%eax;"
    : "=a" (result)
  );
  return result;
}
