bool target_action(Target* t, int stat[])
{
  TEVWH_ELF_PHDR* phdr_note;
  Elf32_Word filesz;

  TRACE_DEBUG(-1, "target_action\n");

  phdr_note = t->phdr_note;
  CHECK_BEGIN(SCAN, phdr_note, !=, 0, -1, void*)
    return false;
  CHECK_END

  filesz = phdr_note->p_filesz;
  /* counters were initialized to zero, real minimum is probably higher */
  if (stat[2] == 0 || filesz < stat[2])
    stat[2] = filesz; /* minimum */
  if (filesz > stat[3])
    stat[3] = filesz; /* maximum */

  TRACE_SCAN(-1, "%s ... p_filesz=%u Ok\n", t->clean_src, filesz);
  return true;
}
