bool target_patch_entry_addr(Target* t)
{
  TRACE_DEBUG(-1, "target_patch_entry_addr\n");
  t->original_entry = t->image.ehdr->e_entry;
  t->image.ehdr->e_entry = target_new_entry_addr(t);
  TRACE_DEBUG(-1, "original_entry=%08x e_entry=%08x\n",
    t->original_entry, t->image.ehdr->e_entry);
  return true; /* this implementation can't fail */
}
