void print_errno(int error, const char* msg, ...)
{
  va_list va;

  if (error != -1)
    fprintf(stderr, "(%d) %s\n", error, strerror(error));
  va_start(va, msg); 
  vfprintf(stderr, msg, va);
  va_end(va); 
}
