		.section .text
_start:		push	$4
		pop	%eax		# eax = 4 = write(2)
		push	$3		# nbytes = three characters
		push	$0x08048001	# buf = magic address
		push	$1		# fd = stdout
		push	%eax		# dummy
		int	$0x80
		add	$4*4,%esp	# remove dummy and 3 arguments
		ret
