		.section .text
		.globl _start
_start:		push	$4
		pop	%eax		# eax = 4 = write(2)
		xor	%ebx,%ebx
		inc	%ebx		# fd = stdout
		mov	$0x08048001,%ecx# buf = magic address
		push	$3		
		pop	%edx		# nbytes = three characters
		int	$0x80

		xor	%eax,%eax
		inc	%eax		# eax = 1 = exit(2)
		xor	%ebx,%ebx	# return code
		int	$0x80
