#!/bin/sh
shell=$( ${TEVWH_PATH_SED} 1q \
	${TEVWH_OUT}/scanner/segment_padding/infect )
[ -x "${shell}" ] || exit -1

${TEVWH_TMP}/evil_magic/e_entry ${shell} \
| while read entry_point offset
do
  ${TEVWH_PATH_OBJDUMP} -d "--start-address=0x${entry_point}" \
  	${TEVWH_ASM_OBJDUMP} "${shell}" \
  | src/magic_elf/objdump_format.pl -start_address=${entry_point}
done
