		BITS 32
start:		push	dword 0		; replace with original entry address
		pushf
		pusha

		push	byte 4
		pop	eax		; eax = 4 = write(2)
		xor	ebx,ebx
		inc	ebx		; ebx = 1 = stdout
		mov	ecx,0x08048001	; ecx = magic address
		push	byte 3
		pop	edx		; edx = 3 = three characters
		int	0x80

		popa
		popf
		ret

		push	byte start + 1	; dummy instruction to specify ofs
