		.section .text
		.globl _start
_start:		set	4, %g1			! write
		set	1, %o0			! fd=1
		set	0x10001, %o1		! buf=0x10001
		set	3, %o2			! count=3
		ta	0x10

		set	1, %g1			! exit
		mov	%g0, %o0		! status=0
		ta	0x10
